// This code conforms with the UFC specification version 1.4
// and was automatically generated by FFC version 0.9.0.
//
// This code was generated with the option '-l dolfin' and
// contains DOLFIN-specific wrappers that depend on DOLFIN.

#ifndef __POISSON2D_4_H
#define __POISSON2D_4_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class poisson2d_4_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  poisson2d_4_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson2d_4_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 15;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Reset values
    *values = 0.000000000000000;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Array of basisvalues
    double basisvalues[15] = {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000};
    
    // Declare helper variables
    unsigned int rr = 0;
    unsigned int ss = 0;
    unsigned int tt = 0;
    double tmp5 = 0.000000000000000;
    double tmp6 = 0.000000000000000;
    double tmp7 = 0.000000000000000;
    double tmp0 = (1.000000000000000 + Y + 2.000000000000000*X)/2.000000000000000;
    double tmp1 = (1.000000000000000 - Y)/2.000000000000000;
    double tmp2 = tmp1*tmp1;
    
    // Compute basisvalues
    basisvalues[0] = 1.000000000000000;
    basisvalues[1] = tmp0;
    for (unsigned int r = 1; r < 4; r++)
    {
      rr = (r + 1)*((r + 1) + 1)/2;
      ss = r*(r + 1)/2;
      tt = (r - 1)*((r - 1) + 1)/2;
      tmp5 = (1.000000000000000 + 2.000000000000000*r)/(1.000000000000000 + r);
      basisvalues[rr] = (basisvalues[ss]*tmp0*tmp5 - basisvalues[tt]*tmp2*r/(1.000000000000000 + r));
    }// end loop over 'r'
    for (unsigned int r = 0; r < 4; r++)
    {
      rr = (r + 1)*(r + 1 + 1)/2 + 1;
      ss = r*(r + 1)/2;
      basisvalues[rr] = basisvalues[ss]*(0.500000000000000 + r + Y*(1.500000000000000 + r));
    }// end loop over 'r'
    for (unsigned int r = 0; r < 3; r++)
    {
      for (unsigned int s = 1; s < 4 - r; s++)
      {
        rr = (r + s + 1)*(r + s + 1 + 1)/2 + s + 1;
        ss = (r + s)*(r + s + 1)/2 + s;
        tt = (r + s - 1)*(r + s - 1 + 1)/2 + s - 1;
        tmp5 = (2.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(3.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)/(2.000000000000000*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        tmp6 = (1.000000000000000 + 4.000000000000000*r*r + 4.000000000000000*r)*(2.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)/(2.000000000000000*(1.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        tmp7 = (1.000000000000000 + s + 2.000000000000000*r)*(3.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*s/((1.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        basisvalues[rr] = (basisvalues[ss]*(tmp6 + Y*tmp5) - basisvalues[tt]*tmp7);
      }// end loop over 's'
    }// end loop over 'r'
    for (unsigned int r = 0; r < 5; r++)
    {
      for (unsigned int s = 0; s < 5 - r; s++)
      {
        rr = (r + s)*(r + s + 1)/2 + s;
        basisvalues[rr] *= std::sqrt((0.500000000000000 + r)*(1.000000000000000 + r + s));
      }// end loop over 's'
    }// end loop over 'r'
    
    // Table(s) of coefficients
    static const double coefficients0[15][15] = \
    {{0.000000000000000, -0.041239304942116, -0.023809523809524, 0.028980029497628, 0.022447834323382, 0.012960263189329, -0.039594258061100, -0.033463255663157, -0.025920526378658, -0.014965222882255, 0.032124725436631, 0.028331344813852, 0.023944356611608, 0.018547218878482, 0.010708241812210},
    {0.000000000000000, 0.041239304942116, -0.023809523809524, 0.028980029497628, -0.022447834323382, 0.012960263189329, 0.039594258061100, -0.033463255663157, 0.025920526378658, -0.014965222882255, 0.032124725436631, -0.028331344813852, 0.023944356611608, -0.018547218878482, 0.010708241812210},
    {0.000000000000000, 0.000000000000000, 0.047619047619048, 0.000000000000000, 0.000000000000000, 0.038880789567987, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.059860891529020, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.053541209061052},
    {0.125707872210942, 0.131965775814772, -0.025396825396825, 0.139104141588614, -0.071833069834824, 0.031104631654390, 0.063350812897760, 0.026770604530526, -0.062209263308779, 0.047888713223216, 0.000000000000000, 0.056662689627705, -0.083805248140628, 0.083462484953168, -0.053541209061052},
    {-0.031426968052735, 0.010997147984564, 0.006349206349206, 0.000000000000000, 0.188561808316413, -0.163299316185545, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.000000000000000, 0.000000000000000, 0.083805248140628, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.043988591938257, 0.126984126984127, 0.000000000000000, 0.035916534917412, 0.155523158271948, 0.000000000000000, 0.000000000000000, 0.103682105514632, -0.011972178305804, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.092736094392409, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.025396825396825, 0.139104141588614, 0.071833069834824, 0.031104631654390, -0.063350812897760, 0.026770604530526, 0.062209263308779, 0.047888713223216, 0.000000000000000, -0.056662689627705, -0.083805248140628, -0.083462484953168, -0.053541209061052},
    {-0.031426968052735, -0.010997147984564, 0.006349206349206, 0.000000000000000, -0.188561808316413, -0.163299316185545, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.000000000000000, 0.000000000000000, 0.083805248140628, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.043988591938257, 0.126984126984127, 0.000000000000000, -0.035916534917412, 0.155523158271948, 0.000000000000000, 0.000000000000000, -0.103682105514632, -0.011972178305804, 0.000000000000000, 0.000000000000000, 0.000000000000000, -0.092736094392409, -0.107082418122104},
    {0.125707872210942, -0.087977183876514, -0.101587301587302, 0.092736094392409, 0.107749604752236, 0.072577473860242, 0.079188516122200, -0.013385302265263, -0.051841052757316, -0.041902624070314, -0.128498901746525, -0.056662689627705, -0.011972178305804, 0.009273609439241, 0.010708241812210},
    {-0.031426968052735, 0.000000000000000, -0.012698412698413, -0.243432247780074, 0.000000000000000, 0.054433105395182, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.192748352619787, 0.000000000000000, -0.023944356611608, 0.000000000000000, 0.010708241812210},
    {0.125707872210942, 0.087977183876514, -0.101587301587302, 0.092736094392409, -0.107749604752236, 0.072577473860242, -0.079188516122200, -0.013385302265263, 0.051841052757316, -0.041902624070314, -0.128498901746525, 0.056662689627705, -0.011972178305804, -0.009273609439241, 0.010708241812210},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.062209263308779, 0.190052438693280, -0.026770604530526, 0.124418526617558, 0.155638317975452, 0.000000000000000, 0.169988068883114, 0.083805248140628, -0.027820828317723, -0.053541209061052},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.062209263308779, -0.190052438693280, -0.026770604530526, -0.124418526617558, 0.155638317975452, 0.000000000000000, -0.169988068883114, 0.083805248140628, 0.027820828317723, -0.053541209061052},
    {0.251415744421883, 0.000000000000000, 0.406349206349206, 0.000000000000000, 0.000000000000000, -0.186627789926337, 0.000000000000000, -0.187394231713682, 0.000000000000000, -0.203527031198668, 0.000000000000000, 0.000000000000000, -0.167610496281256, 0.000000000000000, 0.107082418122104}};
    
    // Compute value(s).
    for (unsigned int r = 0; r < 15; r++)
    {
      *values += coefficients0[dof][r]*basisvalues[r];
    }// end loop over 'r'
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.000000000000000;
    
    // Loop dofs and call evaluate_basis.
    for (unsigned int r = 0; r < 15; r++)
    {
      evaluate_basis(r, &dof_values, coordinates, c);
      values[r] = dof_values;
    }// end loop over 'r'
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    const double K_00 =  J_11 / detJ;
    const double K_01 = -J_01 / detJ;
    const double K_10 = -J_10 / detJ;
    const double K_11 =  J_00 / detJ;
    
    // Compute constants
    const double C0 = x[1][0] + x[2][0];
    const double C1 = x[1][1] + x[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (J_01*(C1 - 2.0*coordinates[1]) + J_11*(2.0*coordinates[0] - C0)) / detJ;
    double Y = (J_00*(2.0*coordinates[1] - C1) + J_10*(C0 - 2.0*coordinates[0])) / detJ;
    
    // Compute number of derivatives.
    unsigned int  num_derivatives = 1;
    
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      combinations[row] = new unsigned int [n];
      for (unsigned int col = 0; col < n; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] = {{K_00, K_01}, {K_10, K_11}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.000000000000000;
    }// end loop over 'r'
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Array of basisvalues
    double basisvalues[15] = {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000};
    
    // Declare helper variables
    unsigned int rr = 0;
    unsigned int ss = 0;
    unsigned int tt = 0;
    double tmp5 = 0.000000000000000;
    double tmp6 = 0.000000000000000;
    double tmp7 = 0.000000000000000;
    double tmp0 = (1.000000000000000 + Y + 2.000000000000000*X)/2.000000000000000;
    double tmp1 = (1.000000000000000 - Y)/2.000000000000000;
    double tmp2 = tmp1*tmp1;
    
    // Compute basisvalues
    basisvalues[0] = 1.000000000000000;
    basisvalues[1] = tmp0;
    for (unsigned int r = 1; r < 4; r++)
    {
      rr = (r + 1)*((r + 1) + 1)/2;
      ss = r*(r + 1)/2;
      tt = (r - 1)*((r - 1) + 1)/2;
      tmp5 = (1.000000000000000 + 2.000000000000000*r)/(1.000000000000000 + r);
      basisvalues[rr] = (basisvalues[ss]*tmp0*tmp5 - basisvalues[tt]*tmp2*r/(1.000000000000000 + r));
    }// end loop over 'r'
    for (unsigned int r = 0; r < 4; r++)
    {
      rr = (r + 1)*(r + 1 + 1)/2 + 1;
      ss = r*(r + 1)/2;
      basisvalues[rr] = basisvalues[ss]*(0.500000000000000 + r + Y*(1.500000000000000 + r));
    }// end loop over 'r'
    for (unsigned int r = 0; r < 3; r++)
    {
      for (unsigned int s = 1; s < 4 - r; s++)
      {
        rr = (r + s + 1)*(r + s + 1 + 1)/2 + s + 1;
        ss = (r + s)*(r + s + 1)/2 + s;
        tt = (r + s - 1)*(r + s - 1 + 1)/2 + s - 1;
        tmp5 = (2.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(3.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)/(2.000000000000000*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        tmp6 = (1.000000000000000 + 4.000000000000000*r*r + 4.000000000000000*r)*(2.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)/(2.000000000000000*(1.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        tmp7 = (1.000000000000000 + s + 2.000000000000000*r)*(3.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*s/((1.000000000000000 + 2.000000000000000*r + 2.000000000000000*s)*(1.000000000000000 + s)*(2.000000000000000 + s + 2.000000000000000*r));
        basisvalues[rr] = (basisvalues[ss]*(tmp6 + Y*tmp5) - basisvalues[tt]*tmp7);
      }// end loop over 's'
    }// end loop over 'r'
    for (unsigned int r = 0; r < 5; r++)
    {
      for (unsigned int s = 0; s < 5 - r; s++)
      {
        rr = (r + s)*(r + s + 1)/2 + s;
        basisvalues[rr] *= std::sqrt((0.500000000000000 + r)*(1.000000000000000 + r + s));
      }// end loop over 's'
    }// end loop over 'r'
    
    // Table(s) of coefficients
    static const double coefficients0[15][15] = \
    {{0.000000000000000, -0.041239304942116, -0.023809523809524, 0.028980029497628, 0.022447834323382, 0.012960263189329, -0.039594258061100, -0.033463255663157, -0.025920526378658, -0.014965222882255, 0.032124725436631, 0.028331344813852, 0.023944356611608, 0.018547218878482, 0.010708241812210},
    {0.000000000000000, 0.041239304942116, -0.023809523809524, 0.028980029497628, -0.022447834323382, 0.012960263189329, 0.039594258061100, -0.033463255663157, 0.025920526378658, -0.014965222882255, 0.032124725436631, -0.028331344813852, 0.023944356611608, -0.018547218878482, 0.010708241812210},
    {0.000000000000000, 0.000000000000000, 0.047619047619048, 0.000000000000000, 0.000000000000000, 0.038880789567987, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.059860891529020, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.053541209061052},
    {0.125707872210942, 0.131965775814772, -0.025396825396825, 0.139104141588614, -0.071833069834824, 0.031104631654390, 0.063350812897760, 0.026770604530526, -0.062209263308779, 0.047888713223216, 0.000000000000000, 0.056662689627705, -0.083805248140628, 0.083462484953168, -0.053541209061052},
    {-0.031426968052735, 0.010997147984564, 0.006349206349206, 0.000000000000000, 0.188561808316413, -0.163299316185545, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.000000000000000, 0.000000000000000, 0.083805248140628, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.043988591938257, 0.126984126984127, 0.000000000000000, 0.035916534917412, 0.155523158271948, 0.000000000000000, 0.000000000000000, 0.103682105514632, -0.011972178305804, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.092736094392409, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.025396825396825, 0.139104141588614, 0.071833069834824, 0.031104631654390, -0.063350812897760, 0.026770604530526, 0.062209263308779, 0.047888713223216, 0.000000000000000, -0.056662689627705, -0.083805248140628, -0.083462484953168, -0.053541209061052},
    {-0.031426968052735, -0.010997147984564, 0.006349206349206, 0.000000000000000, -0.188561808316413, -0.163299316185545, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.000000000000000, 0.000000000000000, 0.083805248140628, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.043988591938257, 0.126984126984127, 0.000000000000000, -0.035916534917412, 0.155523158271948, 0.000000000000000, 0.000000000000000, -0.103682105514632, -0.011972178305804, 0.000000000000000, 0.000000000000000, 0.000000000000000, -0.092736094392409, -0.107082418122104},
    {0.125707872210942, -0.087977183876514, -0.101587301587302, 0.092736094392409, 0.107749604752236, 0.072577473860242, 0.079188516122200, -0.013385302265263, -0.051841052757316, -0.041902624070314, -0.128498901746525, -0.056662689627705, -0.011972178305804, 0.009273609439241, 0.010708241812210},
    {-0.031426968052735, 0.000000000000000, -0.012698412698413, -0.243432247780074, 0.000000000000000, 0.054433105395182, 0.000000000000000, 0.093697115856841, 0.000000000000000, -0.041902624070314, 0.192748352619787, 0.000000000000000, -0.023944356611608, 0.000000000000000, 0.010708241812210},
    {0.125707872210942, 0.087977183876514, -0.101587301587302, 0.092736094392409, -0.107749604752236, 0.072577473860242, -0.079188516122200, -0.013385302265263, 0.051841052757316, -0.041902624070314, -0.128498901746525, 0.056662689627705, -0.011972178305804, -0.009273609439241, 0.010708241812210},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.062209263308779, 0.190052438693280, -0.026770604530526, 0.124418526617558, 0.155638317975452, 0.000000000000000, 0.169988068883114, 0.083805248140628, -0.027820828317723, -0.053541209061052},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.062209263308779, -0.190052438693280, -0.026770604530526, -0.124418526617558, 0.155638317975452, 0.000000000000000, -0.169988068883114, 0.083805248140628, 0.027820828317723, -0.053541209061052},
    {0.251415744421883, 0.000000000000000, 0.406349206349206, 0.000000000000000, 0.000000000000000, -0.186627789926337, 0.000000000000000, -0.187394231713682, 0.000000000000000, -0.203527031198668, 0.000000000000000, 0.000000000000000, -0.167610496281256, 0.000000000000000, 0.107082418122104}};
    
    // Tables of derivatives of the polynomial base (transpose).
    static const double dmats0[15][15] = \
    {{0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {4.898979485566356, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 9.486832980505138, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {4.000000000000003, 0.000000000000000, 7.071067811865476, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {5.291502622129181, 0.000000000000000, -2.993325909419157, 13.662601021279468, 0.000000000000000, 0.611010092660778, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 4.381780460041328, 0.000000000000000, 0.000000000000000, 12.521980673998829, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {3.464101615137757, 0.000000000000000, 7.838367176906172, 0.000000000000000, 0.000000000000000, 8.400000000000004, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 10.954451150103328, 0.000000000000000, 0.000000000000000, -3.833259389999646, 0.000000000000000, 17.748239349298849, 0.000000000000000, 0.553283335172492, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {4.732863826479690, 0.000000000000000, 3.346640106136305, 4.364357804719850, 0.000000000000000, -5.074680379332384, 0.000000000000000, 17.008401285415225, 0.000000000000000, 1.521277658511328, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 2.449489742783177, 0.000000000000000, 0.000000000000000, 9.142857142857148, 0.000000000000000, 0.000000000000000, 0.000000000000000, 14.846149779161806, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {3.098386676965931, 0.000000000000000, 7.668115805072329, 0.000000000000000, 0.000000000000000, 10.733126291998984, 0.000000000000000, 0.000000000000000, 0.000000000000000, 9.295160030897801, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000}};
    
    static const double dmats1[15][15] = \
    {{0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.449489742783178, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {4.242640687119287, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.581988897471612, 4.743416490252570, -0.912870929175276, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.000000000000003, 6.123724356957946, 3.535533905932739, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {-2.309401076758501, 0.000000000000000, 8.164965809277261, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.645751311064592, 5.184592558726290, -1.496662954709578, 6.831300510639734, -1.058300524425837, 0.305505046330390, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.236067977499790, 2.190890230020665, 2.529822128134704, 8.082903768654765, 6.260990336999419, -1.807392228230128, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {1.732050807568880, -5.091168824543140, 3.919183588453085, 0.000000000000000, 9.699484522385713, 4.200000000000003, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {5.000000000000001, 0.000000000000000, -2.828427124746189, 0.000000000000000, 0.000000000000000, 12.124355652982139, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.683281572999747, 5.477225575051664, -1.897366596101030, 7.423074889580906, -1.916629694999818, 0.663940002206988, 8.874119674649425, -1.071428571428572, 0.276641667586245, -0.095831484749991, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.366431913239847, 2.898275349237888, 1.673320053068153, 2.182178902359927, 5.747048932153910, -2.537340189666186, 10.062305898749058, 8.504200642707612, -2.195775164134199, 0.760638829255663, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {2.000000000000000, 1.224744871391588, 3.535533905932740, -7.377111135633178, 4.571428571428575, 1.649572197684644, 0.000000000000000, 11.499778169998919, 7.423074889580904, -2.571428571428571, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {1.549193338482961, 6.640783086353594, 3.834057902536164, 0.000000000000000, -6.196773353931865, 5.366563145999488, 0.000000000000000, 0.000000000000000, 13.416407864998741, 4.647580015448900, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {-3.577708763999667, 0.000000000000000, 8.854377448471467, 0.000000000000000, 0.000000000000000, -3.098386676965937, 0.000000000000000, 0.000000000000000, 0.000000000000000, 16.099689437998492, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      derivatives[r] = 0.000000000000000;
    }// end loop over 'r'
    
    // Declare derivative matrix (of polynomial basis).
    double dmats[15][15] = \
    {{1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000}};
    
    // Declare (auxiliary) derivative matrix (of polynomial basis).
    double dmats_old[15][15] = \
    {{1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000, 0.000000000000000},
    {0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 0.000000000000000, 1.000000000000000}};
    
    // Loop possible derivatives.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      // Resetting dmats values to compute next derivative.
      for (unsigned int t = 0; t < 15; t++)
      {
        for (unsigned int u = 0; u < 15; u++)
        {
          dmats[t][u] = 0.000000000000000;
          if (t == u)
          {
          dmats[t][u] = 1.000000000000000;
          }
          
        }// end loop over 'u'
      }// end loop over 't'
      
      // Looping derivative order to generate dmats.
      for (unsigned int s = 0; s < n; s++)
      {
        // Updating dmats_old with new values and resetting dmats.
        for (unsigned int t = 0; t < 15; t++)
        {
          for (unsigned int u = 0; u < 15; u++)
          {
            dmats_old[t][u] = dmats[t][u];
            dmats[t][u] = 0.000000000000000;
          }// end loop over 'u'
        }// end loop over 't'
        
        // Update dmats using an inner product.
        if (combinations[r][s] == 0)
        {
        for (unsigned int t = 0; t < 15; t++)
        {
          for (unsigned int u = 0; u < 15; u++)
          {
            for (unsigned int tu = 0; tu < 15; tu++)
            {
              dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
            }// end loop over 'tu'
          }// end loop over 'u'
        }// end loop over 't'
        }
        
        if (combinations[r][s] == 1)
        {
        for (unsigned int t = 0; t < 15; t++)
        {
          for (unsigned int u = 0; u < 15; u++)
          {
            for (unsigned int tu = 0; tu < 15; tu++)
            {
              dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
            }// end loop over 'tu'
          }// end loop over 'u'
        }// end loop over 't'
        }
        
      }// end loop over 's'
      for (unsigned int s = 0; s < 15; s++)
      {
        for (unsigned int t = 0; t < 15; t++)
        {
          derivatives[r] += coefficients0[dof][s]*dmats[s][t]*basisvalues[t];
        }// end loop over 't'
      }// end loop over 's'
    }// end loop over 'r'
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      delete [] combinations[r];
      delete [] transform[r];
    }// end loop over 'r'
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    // Compute number of derivatives.
    unsigned int  num_derivatives = 1;
    
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 2;
    }// end loop over 'r'
    
    // Helper variable to hold values of a single dof.
    double *dof_values = new double [num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      dof_values[r] = 0.000000000000000;
    }// end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 15; r++)
    {
      evaluate_basis_derivatives(r, n, dof_values, coordinates, c);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      }// end loop over 's'
    }// end loop over 'r'
    
    // Delete pointer.
    delete [] dof_values;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    
    const double * const * x = c.coordinates;
    switch (i)
    {
    case 0:
      {
        y[0] = x[0][0];
      y[1] = x[0][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = x[1][0];
      y[1] = x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = x[2][0];
      y[1] = x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = 0.75*x[1][0] + 0.25*x[2][0];
      y[1] = 0.75*x[1][1] + 0.25*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 4:
      {
        y[0] = 0.5*x[1][0] + 0.5*x[2][0];
      y[1] = 0.5*x[1][1] + 0.5*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 5:
      {
        y[0] = 0.25*x[1][0] + 0.75*x[2][0];
      y[1] = 0.25*x[1][1] + 0.75*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 6:
      {
        y[0] = 0.75*x[0][0] + 0.25*x[2][0];
      y[1] = 0.75*x[0][1] + 0.25*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 7:
      {
        y[0] = 0.5*x[0][0] + 0.5*x[2][0];
      y[1] = 0.5*x[0][1] + 0.5*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 8:
      {
        y[0] = 0.25*x[0][0] + 0.75*x[2][0];
      y[1] = 0.25*x[0][1] + 0.75*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 9:
      {
        y[0] = 0.75*x[0][0] + 0.25*x[1][0];
      y[1] = 0.75*x[0][1] + 0.25*x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 10:
      {
        y[0] = 0.5*x[0][0] + 0.5*x[1][0];
      y[1] = 0.5*x[0][1] + 0.5*x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 11:
      {
        y[0] = 0.25*x[0][0] + 0.75*x[1][0];
      y[1] = 0.25*x[0][1] + 0.75*x[1][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 12:
      {
        y[0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
      y[1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 13:
      {
        y[0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
      y[1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 14:
      {
        y[0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
      y[1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    // Declare variables for result of evaluation
    double vals[1];
    
    // Declare variable for physical coordinates
    double y[2];
    
    const double * const * x = c.coordinates;
    y[0] = x[0][0];
    y[1] = x[0][1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = x[1][0];
    y[1] = x[1][1];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = x[2][0];
    y[1] = x[2][1];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = 0.75*x[1][0] + 0.25*x[2][0];
    y[1] = 0.75*x[1][1] + 0.25*x[2][1];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.5*x[1][0] + 0.5*x[2][0];
    y[1] = 0.5*x[1][1] + 0.5*x[2][1];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.25*x[1][0] + 0.75*x[2][0];
    y[1] = 0.25*x[1][1] + 0.75*x[2][1];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
    y[0] = 0.75*x[0][0] + 0.25*x[2][0];
    y[1] = 0.75*x[0][1] + 0.25*x[2][1];
    f.evaluate(vals, y, c);
    values[6] = vals[0];
    y[0] = 0.5*x[0][0] + 0.5*x[2][0];
    y[1] = 0.5*x[0][1] + 0.5*x[2][1];
    f.evaluate(vals, y, c);
    values[7] = vals[0];
    y[0] = 0.25*x[0][0] + 0.75*x[2][0];
    y[1] = 0.25*x[0][1] + 0.75*x[2][1];
    f.evaluate(vals, y, c);
    values[8] = vals[0];
    y[0] = 0.75*x[0][0] + 0.25*x[1][0];
    y[1] = 0.75*x[0][1] + 0.25*x[1][1];
    f.evaluate(vals, y, c);
    values[9] = vals[0];
    y[0] = 0.5*x[0][0] + 0.5*x[1][0];
    y[1] = 0.5*x[0][1] + 0.5*x[1][1];
    f.evaluate(vals, y, c);
    values[10] = vals[0];
    y[0] = 0.25*x[0][0] + 0.75*x[1][0];
    y[1] = 0.25*x[0][1] + 0.75*x[1][1];
    f.evaluate(vals, y, c);
    values[11] = vals[0];
    y[0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    y[1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    f.evaluate(vals, y, c);
    values[12] = vals[0];
    y[0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    y[1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    f.evaluate(vals, y, c);
    values[13] = vals[0];
    y[0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    y[1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
    f.evaluate(vals, y, c);
    values[14] = vals[0];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 0;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson2d_4_dof_map_0: public ufc::dof_map
{
private:

  unsigned int _global_dimension;

public:

  /// Constructor
  poisson2d_4_dof_map_0() : ufc::dof_map()
  {
    _global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson2d_4_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dofmap for FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return true;
        break;
      }
    case 2:
      {
        return true;
        break;
      }
    }
    
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    _global_dimension = m.num_entities[0] + 3*m.num_entities[1] + 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return _global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 15;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 15;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 5;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 3;
        break;
      }
    case 2:
      {
        return 3;
        break;
      }
    }
    
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    unsigned int offset = 0;
    
    dofs[0] = offset + c.entity_indices[0][0];
    dofs[1] = offset + c.entity_indices[0][1];
    dofs[2] = offset + c.entity_indices[0][2];
    offset += m.num_entities[0];
    dofs[3] = offset + 3*c.entity_indices[1][0];
    dofs[4] = offset + 3*c.entity_indices[1][0] + 1;
    dofs[5] = offset + 3*c.entity_indices[1][0] + 2;
    dofs[6] = offset + 3*c.entity_indices[1][1];
    dofs[7] = offset + 3*c.entity_indices[1][1] + 1;
    dofs[8] = offset + 3*c.entity_indices[1][1] + 2;
    dofs[9] = offset + 3*c.entity_indices[1][2];
    dofs[10] = offset + 3*c.entity_indices[1][2] + 1;
    dofs[11] = offset + 3*c.entity_indices[1][2] + 2;
    offset += 3*m.num_entities[1];
    dofs[12] = offset + 3*c.entity_indices[2][0];
    dofs[13] = offset + 3*c.entity_indices[2][0] + 1;
    dofs[14] = offset + 3*c.entity_indices[2][0] + 2;
    offset += 3*m.num_entities[2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 6;
      dofs[3] = 7;
      dofs[4] = 8;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 9;
      dofs[3] = 10;
      dofs[4] = 11;
        break;
      }
    }
    
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    if (d > 2)
    {
    throw std::runtime_error("d is larger than dimension (2)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 2)
      {
      throw std::runtime_error("i is larger than number of entities (2)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 3;
        dofs[1] = 4;
        dofs[2] = 5;
          break;
        }
      case 1:
        {
          dofs[0] = 6;
        dofs[1] = 7;
        dofs[2] = 8;
          break;
        }
      case 2:
        {
          dofs[0] = 9;
        dofs[1] = 10;
        dofs[2] = 11;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 12;
      dofs[1] = 13;
      dofs[2] = 14;
        break;
      }
    }
    
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.75*x[1][0] + 0.25*x[2][0];
    coordinates[3][1] = 0.75*x[1][1] + 0.25*x[2][1];
    coordinates[4][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.25*x[1][0] + 0.75*x[2][0];
    coordinates[5][1] = 0.25*x[1][1] + 0.75*x[2][1];
    coordinates[6][0] = 0.75*x[0][0] + 0.25*x[2][0];
    coordinates[6][1] = 0.75*x[0][1] + 0.25*x[2][1];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][0] = 0.25*x[0][0] + 0.75*x[2][0];
    coordinates[8][1] = 0.25*x[0][1] + 0.75*x[2][1];
    coordinates[9][0] = 0.75*x[0][0] + 0.25*x[1][0];
    coordinates[9][1] = 0.75*x[0][1] + 0.25*x[1][1];
    coordinates[10][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[10][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[11][0] = 0.25*x[0][0] + 0.75*x[1][0];
    coordinates[11][1] = 0.25*x[0][1] + 0.75*x[1][1];
    coordinates[12][0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    coordinates[12][1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    coordinates[13][0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    coordinates[13][1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    coordinates[14][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    coordinates[14][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 0;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson2d_4_cell_integral_0_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson2d_4_cell_integral_0_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson2d_4_cell_integral_0_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      12
    // Number of operations (multiply-add pairs) for geometry tensor:    8
    // Number of operations (multiply-add pairs) for tensor contraction: 654
    // Total number of operations (multiply-add pairs):                  674
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    const double K_00 =  J_11 / detJ;
    const double K_01 = -J_01 / detJ;
    const double K_10 = -J_10 / detJ;
    const double K_11 =  J_00 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0_0 = det*(K_00*K_00 + K_01*K_01);
    const double G0_0_1 = det*(K_00*K_10 + K_01*K_11);
    const double G0_1_0 = det*(K_10*K_00 + K_11*K_01);
    const double G0_1_1 = det*(K_10*K_10 + K_11*K_11);
    
    // Compute element tensor
    A[0] = 0.373015873015873*G0_0_0 + 0.373015873015873*G0_0_1 + 0.373015873015873*G0_1_0 + 0.373015873015872*G0_1_1;
    A[1] = 0.056613756613756*G0_0_0 + 0.056613756613756*G0_1_0;
    A[2] = 0.056613756613757*G0_0_1 + 0.056613756613757*G0_1_1;
    A[3] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[4] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[5] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328043*G0_1_0 + 0.042328042328042*G0_1_1;
    A[6] = 0.126984126984127*G0_0_0 - 0.651851851851851*G0_0_1 + 0.126984126984127*G0_1_0 - 0.651851851851851*G0_1_1;
    A[7] = 0.042328042328042*G0_0_0 + 0.467724867724867*G0_0_1 + 0.042328042328042*G0_1_0 + 0.467724867724867*G0_1_1;
    A[8] = (-0.042328042328042)*G0_0_0 - 0.245502645502645*G0_0_1 - 0.042328042328043*G0_1_0 - 0.245502645502646*G0_1_1;
    A[9] = (-0.651851851851851)*G0_0_0 + 0.126984126984127*G0_0_1 - 0.651851851851851*G0_1_0 + 0.126984126984127*G0_1_1;
    A[10] = 0.467724867724867*G0_0_0 + 0.042328042328042*G0_0_1 + 0.467724867724867*G0_1_0 + 0.042328042328042*G0_1_1;
    A[11] = (-0.245502645502645)*G0_0_0 - 0.042328042328042*G0_0_1 - 0.245502645502645*G0_1_0 - 0.042328042328042*G0_1_1;
    A[12] = (-0.084656084656085)*G0_0_0 - 0.084656084656085*G0_0_1 - 0.084656084656085*G0_1_0 - 0.084656084656085*G0_1_1;
    A[13] = (-0.084656084656084)*G0_0_0 - 0.084656084656085*G0_0_1 - 0.084656084656084*G0_1_0 - 0.084656084656085*G0_1_1;
    A[14] = (-0.084656084656084)*G0_0_0 - 0.084656084656084*G0_0_1 - 0.084656084656084*G0_1_0 - 0.084656084656084*G0_1_1;
    A[15] = 0.056613756613756*G0_0_0 + 0.056613756613756*G0_0_1;
    A[16] = 0.373015873015873*G0_0_0;
    A[17] = (-0.056613756613757)*G0_0_1;
    A[18] = 0.126984126984126*G0_0_0 + 0.778835978835978*G0_0_1;
    A[19] = 0.042328042328042*G0_0_0 - 0.425396825396825*G0_0_1;
    A[20] = (-0.042328042328042)*G0_0_0 + 0.203174603174603*G0_0_1;
    A[21] = 0.042328042328042*G0_0_0;
    A[22] = 0.042328042328042*G0_0_0;
    A[23] = 0.042328042328042*G0_0_0;
    A[24] = (-0.245502645502645)*G0_0_0 - 0.203174603174603*G0_0_1;
    A[25] = 0.467724867724868*G0_0_0 + 0.425396825396825*G0_0_1;
    A[26] = (-0.651851851851851)*G0_0_0 - 0.778835978835978*G0_0_1;
    A[27] = (-0.084656084656085)*G0_0_0;
    A[28] = (-0.084656084656084)*G0_0_0;
    A[29] = (-0.084656084656085)*G0_0_0;
    A[30] = 0.056613756613757*G0_1_0 + 0.056613756613757*G0_1_1;
    A[31] = (-0.056613756613757)*G0_1_0;
    A[32] = 0.373015873015873*G0_1_1;
    A[33] = 0.203174603174603*G0_1_0 - 0.042328042328043*G0_1_1;
    A[34] = (-0.425396825396824)*G0_1_0 + 0.042328042328043*G0_1_1;
    A[35] = 0.778835978835979*G0_1_0 + 0.126984126984127*G0_1_1;
    A[36] = (-0.203174603174603)*G0_1_0 - 0.245502645502646*G0_1_1;
    A[37] = 0.425396825396825*G0_1_0 + 0.467724867724868*G0_1_1;
    A[38] = (-0.778835978835979)*G0_1_0 - 0.651851851851852*G0_1_1;
    A[39] = 0.042328042328043*G0_1_1;
    A[40] = 0.042328042328042*G0_1_1;
    A[41] = 0.042328042328042*G0_1_1;
    A[42] = (-0.084656084656085)*G0_1_1;
    A[43] = (-0.084656084656085)*G0_1_1;
    A[44] = (-0.084656084656084)*G0_1_1;
    A[45] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[46] = 0.126984126984126*G0_0_0 + 0.778835978835978*G0_1_0;
    A[47] = 0.203174603174603*G0_0_1 - 0.042328042328043*G0_1_1;
    A[48] = 1.828571428571424*G0_0_0 + 1.049735449735447*G0_0_1 + 1.049735449735447*G0_1_0 + 2.099470899470897*G0_1_1;
    A[49] = (-0.660317460317458)*G0_0_0 + 0.821164021164020*G0_0_1 - 0.296296296296295*G0_1_0 - 0.761904761904762*G0_1_1;
    A[50] = 0.338624338624337*G0_0_0 - 0.440211640211640*G0_0_1 + 0.101587301587300*G0_1_0 + 0.338624338624338*G0_1_1;
    A[51] = (-0.203174603174602)*G0_0_0 - 0.101587301587301*G0_0_1 - 0.101587301587301*G0_1_0;
    A[52] = (-0.152380952380953)*G0_0_0 - 0.042328042328043*G0_0_1 - 0.042328042328043*G0_1_0;
    A[53] = (-0.338624338624337)*G0_0_0 - 0.101587301587301*G0_0_1 - 0.101587301587301*G0_1_0;
    A[54] = (-0.135449735449735)*G0_0_0 - 0.237037037037035*G0_0_1 - 0.237037037037037*G0_1_0 - 0.338624338624338*G0_1_1;
    A[55] = 0.101587301587301*G0_0_0 + 0.465608465608464*G0_0_1 + 0.465608465608466*G0_1_0 + 0.761904761904761*G0_1_1;
    A[56] = (-0.135449735449735)*G0_0_0 - 1.049735449735447*G0_0_1 - 1.049735449735449*G0_1_0 - 2.099470899470897*G0_1_1;
    A[57] = 0.812698412698408*G0_0_0 + 0.338624338624335*G0_0_1 + 0.338624338624338*G0_1_0;
    A[58] = (-2.438095238095232)*G0_0_0 - 1.286772486772485*G0_0_1 - 1.286772486772484*G0_1_0;
    A[59] = 0.812698412698412*G0_0_0 + 0.338624338624340*G0_0_1 + 0.338624338624339*G0_1_0;
    A[60] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[61] = 0.042328042328042*G0_0_0 - 0.425396825396825*G0_1_0;
    A[62] = (-0.425396825396824)*G0_0_1 + 0.042328042328043*G0_1_1;
    A[63] = (-0.660317460317458)*G0_0_0 - 0.296296296296295*G0_0_1 + 0.821164021164020*G0_1_0 - 0.761904761904762*G0_1_1;
    A[64] = 2.450793650793646*G0_0_0 + 0.971428571428569*G0_0_1 + 0.971428571428569*G0_1_0 + 2.450793650793646*G0_1_1;
    A[65] = (-0.761904761904759)*G0_0_0 + 0.821164021164022*G0_0_1 - 0.296296296296295*G0_1_0 - 0.660317460317459*G0_1_1;
    A[66] = (-0.152380952380953)*G0_0_0 - 0.110052910052909*G0_0_1 - 0.110052910052910*G0_1_0 - 0.067724867724868*G0_1_1;
    A[67] = 0.393650793650795*G0_0_0 + 0.196825396825397*G0_0_1 + 0.196825396825398*G0_1_0 + 0.050793650793652*G0_1_1;
    A[68] = 0.761904761904759*G0_0_0 + 0.296296296296295*G0_0_1 + 0.296296296296295*G0_1_0 - 0.067724867724869*G0_1_1;
    A[69] = (-0.067724867724868)*G0_0_0 - 0.110052910052912*G0_0_1 - 0.110052910052909*G0_1_0 - 0.152380952380952*G0_1_1;
    A[70] = 0.050793650793650*G0_0_0 + 0.196825396825396*G0_0_1 + 0.196825396825395*G0_1_0 + 0.393650793650793*G0_1_1;
    A[71] = (-0.067724867724867)*G0_0_0 + 0.296296296296295*G0_0_1 + 0.296296296296297*G0_1_0 + 0.761904761904762*G0_1_1;
    A[72] = 0.406349206349207*G0_0_0 + 0.457142857142859*G0_0_1 + 0.457142857142855*G0_1_0 + 0.406349206349204*G0_1_1;
    A[73] = 0.406349206349205*G0_0_0 - 1.168253968253964*G0_0_1 - 1.168253968253965*G0_1_0 - 2.844444444444439*G0_1_1;
    A[74] = (-2.844444444444441)*G0_0_0 - 1.168253968253970*G0_0_1 - 1.168253968253967*G0_1_0 + 0.406349206349205*G0_1_1;
    A[75] = 0.042328042328042*G0_0_0 + 0.042328042328043*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[76] = (-0.042328042328042)*G0_0_0 + 0.203174603174603*G0_1_0;
    A[77] = 0.778835978835979*G0_0_1 + 0.126984126984127*G0_1_1;
    A[78] = 0.338624338624337*G0_0_0 + 0.101587301587300*G0_0_1 - 0.440211640211640*G0_1_0 + 0.338624338624338*G0_1_1;
    A[79] = (-0.761904761904759)*G0_0_0 - 0.296296296296295*G0_0_1 + 0.821164021164022*G0_1_0 - 0.660317460317459*G0_1_1;
    A[80] = 2.099470899470899*G0_0_0 + 1.049735449735450*G0_0_1 + 1.049735449735450*G0_1_0 + 1.828571428571428*G0_1_1;
    A[81] = (-0.338624338624338)*G0_0_0 - 0.237037037037037*G0_0_1 - 0.237037037037037*G0_1_0 - 0.135449735449735*G0_1_1;
    A[82] = 0.761904761904761*G0_0_0 + 0.465608465608465*G0_0_1 + 0.465608465608465*G0_1_0 + 0.101587301587300*G0_1_1;
    A[83] = (-2.099470899470899)*G0_0_0 - 1.049735449735450*G0_0_1 - 1.049735449735449*G0_1_0 - 0.135449735449735*G0_1_1;
    A[84] = (-0.101587301587301)*G0_0_1 - 0.101587301587301*G0_1_0 - 0.203174603174604*G0_1_1;
    A[85] = (-0.042328042328042)*G0_0_1 - 0.042328042328042*G0_1_0 - 0.152380952380952*G0_1_1;
    A[86] = (-0.101587301587301)*G0_0_1 - 0.101587301587302*G0_1_0 - 0.338624338624339*G0_1_1;
    A[87] = 0.338624338624338*G0_0_1 + 0.338624338624339*G0_1_0 + 0.812698412698414*G0_1_1;
    A[88] = 0.338624338624337*G0_0_1 + 0.338624338624337*G0_1_0 + 0.812698412698411*G0_1_1;
    A[89] = (-1.286772486772486)*G0_0_1 - 1.286772486772487*G0_1_0 - 2.438095238095237*G0_1_1;
    A[90] = 0.126984126984127*G0_0_0 + 0.126984126984127*G0_0_1 - 0.651851851851851*G0_1_0 - 0.651851851851851*G0_1_1;
    A[91] = 0.042328042328042*G0_0_0;
    A[92] = (-0.203174603174603)*G0_0_1 - 0.245502645502646*G0_1_1;
    A[93] = (-0.203174603174602)*G0_0_0 - 0.101587301587301*G0_0_1 - 0.101587301587301*G0_1_0;
    A[94] = (-0.152380952380953)*G0_0_0 - 0.110052910052910*G0_0_1 - 0.110052910052909*G0_1_0 - 0.067724867724868*G0_1_1;
    A[95] = (-0.338624338624338)*G0_0_0 - 0.237037037037037*G0_0_1 - 0.237037037037037*G0_1_0 - 0.135449735449735*G0_1_1;
    A[96] = 1.828571428571427*G0_0_0 + 0.778835978835979*G0_0_1 + 0.778835978835979*G0_1_0 + 1.828571428571427*G0_1_1;
    A[97] = (-0.660317460317459)*G0_0_0 - 1.481481481481481*G0_0_1 - 0.364021164021163*G0_1_0 - 1.947089947089946*G0_1_1;
    A[98] = 0.338624338624338*G0_0_0 + 0.778835978835978*G0_0_1 + 0.237037037037037*G0_1_0 + 1.015873015873016*G0_1_1;
    A[99] = (-0.135449735449735)*G0_0_0 + 0.914285714285713*G0_0_1 + 0.914285714285713*G0_1_0 - 0.135449735449736*G0_1_1;
    A[100] = 0.101587301587301*G0_0_0 - 0.364021164021165*G0_0_1 - 0.364021164021163*G0_1_0 - 0.067724867724868*G0_1_1;
    A[101] = (-0.135449735449734)*G0_0_0 + 0.101587301587302*G0_0_1 + 0.101587301587301*G0_1_0;
    A[102] = (-2.438095238095237)*G0_0_0 - 1.151322751322750*G0_0_1 - 1.151322751322751*G0_1_0 + 0.135449735449737*G0_1_1;
    A[103] = 0.812698412698412*G0_0_0 + 0.474074074074075*G0_0_1 + 0.474074074074073*G0_1_0 + 0.135449735449737*G0_1_1;
    A[104] = 0.812698412698412*G0_0_0 + 0.474074074074073*G0_0_1 + 0.474074074074073*G0_1_0 + 0.135449735449733*G0_1_1;
    A[105] = 0.042328042328042*G0_0_0 + 0.042328042328042*G0_0_1 + 0.467724867724867*G0_1_0 + 0.467724867724867*G0_1_1;
    A[106] = 0.042328042328042*G0_0_0;
    A[107] = 0.425396825396825*G0_0_1 + 0.467724867724868*G0_1_1;
    A[108] = (-0.152380952380953)*G0_0_0 - 0.042328042328043*G0_0_1 - 0.042328042328043*G0_1_0;
    A[109] = 0.393650793650795*G0_0_0 + 0.196825396825398*G0_0_1 + 0.196825396825397*G0_1_0 + 0.050793650793652*G0_1_1;
    A[110] = 0.761904761904761*G0_0_0 + 0.465608465608465*G0_0_1 + 0.465608465608465*G0_1_0 + 0.101587301587300*G0_1_1;
    A[111] = (-0.660317460317459)*G0_0_0 - 0.364021164021164*G0_0_1 - 1.481481481481481*G0_1_0 - 1.947089947089945*G0_1_1;
    A[112] = 2.450793650793646*G0_0_0 + 1.479365079365077*G0_0_1 + 1.479365079365077*G0_1_0 + 2.958730158730156*G0_1_1;
    A[113] = (-0.761904761904761)*G0_0_0 - 1.583068783068781*G0_0_1 - 0.465608465608465*G0_1_0 - 1.947089947089946*G0_1_1;
    A[114] = (-0.067724867724867)*G0_0_0 - 0.364021164021164*G0_0_1 - 0.364021164021163*G0_1_0 + 0.101587301587303*G0_1_1;
    A[115] = 0.050793650793651*G0_0_0 - 0.146031746031744*G0_0_1 - 0.146031746031746*G0_1_0 + 0.050793650793652*G0_1_1;
    A[116] = (-0.067724867724869)*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0;
    A[117] = 0.406349206349207*G0_0_0 + 1.574603174603172*G0_0_1 + 1.574603174603174*G0_1_0 - 0.101587301587303*G0_1_1;
    A[118] = 0.406349206349205*G0_0_0 - 0.050793650793653*G0_0_1 - 0.050793650793651*G0_1_0 - 0.101587301587304*G0_1_1;
    A[119] = (-2.844444444444442)*G0_0_0 - 1.676190476190474*G0_0_1 - 1.676190476190474*G0_1_0 - 0.101587301587299*G0_1_1;
    A[120] = (-0.042328042328042)*G0_0_0 - 0.042328042328043*G0_0_1 - 0.245502645502645*G0_1_0 - 0.245502645502646*G0_1_1;
    A[121] = 0.042328042328042*G0_0_0;
    A[122] = (-0.778835978835979)*G0_0_1 - 0.651851851851852*G0_1_1;
    A[123] = (-0.338624338624337)*G0_0_0 - 0.101587301587301*G0_0_1 - 0.101587301587301*G0_1_0;
    A[124] = 0.761904761904759*G0_0_0 + 0.296296296296295*G0_0_1 + 0.296296296296295*G0_1_0 - 0.067724867724869*G0_1_1;
    A[125] = (-2.099470899470899)*G0_0_0 - 1.049735449735449*G0_0_1 - 1.049735449735450*G0_1_0 - 0.135449735449735*G0_1_1;
    A[126] = 0.338624338624338*G0_0_0 + 0.237037037037037*G0_0_1 + 0.778835978835978*G0_1_0 + 1.015873015873016*G0_1_1;
    A[127] = (-0.761904761904761)*G0_0_0 - 0.465608465608465*G0_0_1 - 1.583068783068781*G0_1_0 - 1.947089947089946*G0_1_1;
    A[128] = 2.099470899470899*G0_0_0 + 1.049735449735449*G0_0_1 + 1.049735449735449*G0_1_0 + 1.828571428571428*G0_1_1;
    A[129] = 0.101587301587301*G0_0_1 + 0.101587301587301*G0_1_0 - 0.135449735449736*G0_1_1;
    A[130] = 0.042328042328042*G0_0_1 + 0.042328042328043*G0_1_0 - 0.067724867724868*G0_1_1;
    A[131] = 0.101587301587302*G0_0_1 + 0.101587301587302*G0_1_0;
    A[132] = (-0.338624338624338)*G0_0_1 - 0.338624338624338*G0_1_0 + 0.135449735449737*G0_1_1;
    A[133] = (-0.338624338624336)*G0_0_1 - 0.338624338624339*G0_1_0 + 0.135449735449737*G0_1_1;
    A[134] = 1.286772486772486*G0_0_1 + 1.286772486772486*G0_1_0 + 0.135449735449734*G0_1_1;
    A[135] = (-0.651851851851851)*G0_0_0 - 0.651851851851851*G0_0_1 + 0.126984126984127*G0_1_0 + 0.126984126984127*G0_1_1;
    A[136] = (-0.245502645502645)*G0_0_0 - 0.203174603174603*G0_1_0;
    A[137] = 0.042328042328043*G0_1_1;
    A[138] = (-0.135449735449735)*G0_0_0 - 0.237037037037037*G0_0_1 - 0.237037037037035*G0_1_0 - 0.338624338624338*G0_1_1;
    A[139] = (-0.067724867724868)*G0_0_0 - 0.110052910052909*G0_0_1 - 0.110052910052912*G0_1_0 - 0.152380952380952*G0_1_1;
    A[140] = (-0.101587301587301)*G0_0_1 - 0.101587301587301*G0_1_0 - 0.203174603174604*G0_1_1;
    A[141] = (-0.135449735449735)*G0_0_0 + 0.914285714285713*G0_0_1 + 0.914285714285713*G0_1_0 - 0.135449735449736*G0_1_1;
    A[142] = (-0.067724867724867)*G0_0_0 - 0.364021164021163*G0_0_1 - 0.364021164021164*G0_1_0 + 0.101587301587302*G0_1_1;
    A[143] = 0.101587301587301*G0_0_1 + 0.101587301587301*G0_1_0 - 0.135449735449736*G0_1_1;
    A[144] = 1.828571428571426*G0_0_0 + 0.778835978835978*G0_0_1 + 0.778835978835978*G0_1_0 + 1.828571428571428*G0_1_1;
    A[145] = (-1.947089947089945)*G0_0_0 - 0.364021164021164*G0_0_1 - 1.481481481481481*G0_1_0 - 0.660317460317461*G0_1_1;
    A[146] = 1.015873015873015*G0_0_0 + 0.237037037037036*G0_0_1 + 0.778835978835979*G0_1_0 + 0.338624338624339*G0_1_1;
    A[147] = 0.135449735449735*G0_0_0 - 1.151322751322750*G0_0_1 - 1.151322751322751*G0_1_0 - 2.438095238095238*G0_1_1;
    A[148] = 0.135449735449734*G0_0_0 + 0.474074074074074*G0_0_1 + 0.474074074074072*G0_1_0 + 0.812698412698412*G0_1_1;
    A[149] = 0.135449735449735*G0_0_0 + 0.474074074074073*G0_0_1 + 0.474074074074075*G0_1_0 + 0.812698412698413*G0_1_1;
    A[150] = 0.467724867724867*G0_0_0 + 0.467724867724867*G0_0_1 + 0.042328042328042*G0_1_0 + 0.042328042328042*G0_1_1;
    A[151] = 0.467724867724868*G0_0_0 + 0.425396825396825*G0_1_0;
    A[152] = 0.042328042328042*G0_1_1;
    A[153] = 0.101587301587301*G0_0_0 + 0.465608465608466*G0_0_1 + 0.465608465608464*G0_1_0 + 0.761904761904761*G0_1_1;
    A[154] = 0.050793650793651*G0_0_0 + 0.196825396825395*G0_0_1 + 0.196825396825396*G0_1_0 + 0.393650793650793*G0_1_1;
    A[155] = (-0.042328042328042)*G0_0_1 - 0.042328042328042*G0_1_0 - 0.152380952380952*G0_1_1;
    A[156] = 0.101587301587300*G0_0_0 - 0.364021164021163*G0_0_1 - 0.364021164021165*G0_1_0 - 0.067724867724868*G0_1_1;
    A[157] = 0.050793650793651*G0_0_0 - 0.146031746031746*G0_0_1 - 0.146031746031744*G0_1_0 + 0.050793650793652*G0_1_1;
    A[158] = 0.042328042328043*G0_0_1 + 0.042328042328042*G0_1_0 - 0.067724867724868*G0_1_1;
    A[159] = (-1.947089947089945)*G0_0_0 - 1.481481481481482*G0_0_1 - 0.364021164021164*G0_1_0 - 0.660317460317461*G0_1_1;
    A[160] = 2.958730158730157*G0_0_0 + 1.479365079365079*G0_0_1 + 1.479365079365079*G0_1_0 + 2.450793650793650*G0_1_1;
    A[161] = (-1.947089947089946)*G0_0_0 - 0.465608465608466*G0_0_1 - 1.583068783068782*G0_1_0 - 0.761904761904761*G0_1_1;
    A[162] = (-0.101587301587300)*G0_0_0 + 1.574603174603175*G0_0_1 + 1.574603174603175*G0_1_0 + 0.406349206349208*G0_1_1;
    A[163] = (-0.101587301587301)*G0_0_0 - 1.676190476190475*G0_0_1 - 1.676190476190475*G0_1_0 - 2.844444444444442*G0_1_1;
    A[164] = (-0.101587301587302)*G0_0_0 - 0.050793650793652*G0_0_1 - 0.050793650793652*G0_1_0 + 0.406349206349205*G0_1_1;
    A[165] = (-0.245502645502645)*G0_0_0 - 0.245502645502645*G0_0_1 - 0.042328042328042*G0_1_0 - 0.042328042328042*G0_1_1;
    A[166] = (-0.651851851851851)*G0_0_0 - 0.778835978835978*G0_1_0;
    A[167] = 0.042328042328042*G0_1_1;
    A[168] = (-0.135449735449735)*G0_0_0 - 1.049735449735449*G0_0_1 - 1.049735449735447*G0_1_0 - 2.099470899470897*G0_1_1;
    A[169] = (-0.067724867724867)*G0_0_0 + 0.296296296296297*G0_0_1 + 0.296296296296295*G0_1_0 + 0.761904761904761*G0_1_1;
    A[170] = (-0.101587301587302)*G0_0_1 - 0.101587301587301*G0_1_0 - 0.338624338624339*G0_1_1;
    A[171] = (-0.135449735449734)*G0_0_0 + 0.101587301587301*G0_0_1 + 0.101587301587302*G0_1_0;
    A[172] = (-0.067724867724869)*G0_0_0 + 0.042328042328042*G0_0_1 + 0.042328042328042*G0_1_0;
    A[173] = 0.101587301587302*G0_0_1 + 0.101587301587302*G0_1_0;
    A[174] = 1.015873015873015*G0_0_0 + 0.778835978835979*G0_0_1 + 0.237037037037036*G0_1_0 + 0.338624338624339*G0_1_1;
    A[175] = (-1.947089947089946)*G0_0_0 - 1.583068783068782*G0_0_1 - 0.465608465608466*G0_1_0 - 0.761904761904761*G0_1_1;
    A[176] = 1.828571428571427*G0_0_0 + 1.049735449735449*G0_0_1 + 1.049735449735449*G0_1_0 + 2.099470899470898*G0_1_1;
    A[177] = 0.135449735449733*G0_0_0 - 0.338624338624340*G0_0_1 - 0.338624338624339*G0_1_0;
    A[178] = 0.135449735449736*G0_0_0 + 1.286772486772486*G0_0_1 + 1.286772486772485*G0_1_0;
    A[179] = 0.135449735449736*G0_0_0 - 0.338624338624337*G0_0_1 - 0.338624338624337*G0_1_0;
    A[180] = (-0.084656084656085)*G0_0_0 - 0.084656084656085*G0_0_1 - 0.084656084656085*G0_1_0 - 0.084656084656085*G0_1_1;
    A[181] = (-0.084656084656085)*G0_0_0;
    A[182] = (-0.084656084656085)*G0_1_1;
    A[183] = 0.812698412698408*G0_0_0 + 0.338624338624338*G0_0_1 + 0.338624338624335*G0_1_0;
    A[184] = 0.406349206349207*G0_0_0 + 0.457142857142855*G0_0_1 + 0.457142857142859*G0_1_0 + 0.406349206349204*G0_1_1;
    A[185] = 0.338624338624339*G0_0_1 + 0.338624338624338*G0_1_0 + 0.812698412698414*G0_1_1;
    A[186] = (-2.438095238095237)*G0_0_0 - 1.151322751322750*G0_0_1 - 1.151322751322750*G0_1_0 + 0.135449735449737*G0_1_1;
    A[187] = 0.406349206349207*G0_0_0 + 1.574603174603174*G0_0_1 + 1.574603174603172*G0_1_0 - 0.101587301587303*G0_1_1;
    A[188] = (-0.338624338624338)*G0_0_1 - 0.338624338624338*G0_1_0 + 0.135449735449737*G0_1_1;
    A[189] = 0.135449735449735*G0_0_0 - 1.151322751322751*G0_0_1 - 1.151322751322750*G0_1_0 - 2.438095238095237*G0_1_1;
    A[190] = (-0.101587301587300)*G0_0_0 + 1.574603174603175*G0_0_1 + 1.574603174603175*G0_1_0 + 0.406349206349208*G0_1_1;
    A[191] = 0.135449735449733*G0_0_0 - 0.338624338624338*G0_0_1 - 0.338624338624340*G0_1_0;
    A[192] = 5.688888888888887*G0_0_0 + 2.844444444444444*G0_0_1 + 2.844444444444444*G0_1_0 + 5.688888888888888*G0_1_1;
    A[193] = (-4.063492063492058)*G0_0_0 - 2.031746031746030*G0_0_1 - 2.031746031746029*G0_1_0 - 0.812698412698411*G0_1_1;
    A[194] = (-0.812698412698413)*G0_0_0 - 2.031746031746031*G0_0_1 - 2.031746031746031*G0_1_0 - 4.063492063492063*G0_1_1;
    A[195] = (-0.084656084656084)*G0_0_0 - 0.084656084656084*G0_0_1 - 0.084656084656085*G0_1_0 - 0.084656084656085*G0_1_1;
    A[196] = (-0.084656084656084)*G0_0_0;
    A[197] = (-0.084656084656085)*G0_1_1;
    A[198] = (-2.438095238095232)*G0_0_0 - 1.286772486772484*G0_0_1 - 1.286772486772485*G0_1_0;
    A[199] = 0.406349206349205*G0_0_0 - 1.168253968253965*G0_0_1 - 1.168253968253964*G0_1_0 - 2.844444444444439*G0_1_1;
    A[200] = 0.338624338624337*G0_0_1 + 0.338624338624337*G0_1_0 + 0.812698412698411*G0_1_1;
    A[201] = 0.812698412698412*G0_0_0 + 0.474074074074073*G0_0_1 + 0.474074074074075*G0_1_0 + 0.135449735449737*G0_1_1;
    A[202] = 0.406349206349205*G0_0_0 - 0.050793650793651*G0_0_1 - 0.050793650793653*G0_1_0 - 0.101587301587304*G0_1_1;
    A[203] = (-0.338624338624339)*G0_0_1 - 0.338624338624336*G0_1_0 + 0.135449735449737*G0_1_1;
    A[204] = 0.135449735449734*G0_0_0 + 0.474074074074072*G0_0_1 + 0.474074074074074*G0_1_0 + 0.812698412698412*G0_1_1;
    A[205] = (-0.101587301587301)*G0_0_0 - 1.676190476190475*G0_0_1 - 1.676190476190475*G0_1_0 - 2.844444444444442*G0_1_1;
    A[206] = 0.135449735449736*G0_0_0 + 1.286772486772485*G0_0_1 + 1.286772486772486*G0_1_0;
    A[207] = (-4.063492063492058)*G0_0_0 - 2.031746031746029*G0_0_1 - 2.031746031746030*G0_1_0 - 0.812698412698411*G0_1_1;
    A[208] = 5.688888888888881*G0_0_0 + 2.844444444444440*G0_0_1 + 2.844444444444441*G0_1_0 + 5.688888888888881*G0_1_1;
    A[209] = (-0.812698412698412)*G0_0_0 + 1.219047619047618*G0_0_1 + 1.219047619047616*G0_1_0 - 0.812698412698411*G0_1_1;
    A[210] = (-0.084656084656084)*G0_0_0 - 0.084656084656084*G0_0_1 - 0.084656084656084*G0_1_0 - 0.084656084656084*G0_1_1;
    A[211] = (-0.084656084656085)*G0_0_0;
    A[212] = (-0.084656084656084)*G0_1_1;
    A[213] = 0.812698412698412*G0_0_0 + 0.338624338624339*G0_0_1 + 0.338624338624340*G0_1_0;
    A[214] = (-2.844444444444441)*G0_0_0 - 1.168253968253967*G0_0_1 - 1.168253968253970*G0_1_0 + 0.406349206349205*G0_1_1;
    A[215] = (-1.286772486772487)*G0_0_1 - 1.286772486772486*G0_1_0 - 2.438095238095237*G0_1_1;
    A[216] = 0.812698412698412*G0_0_0 + 0.474074074074073*G0_0_1 + 0.474074074074073*G0_1_0 + 0.135449735449733*G0_1_1;
    A[217] = (-2.844444444444442)*G0_0_0 - 1.676190476190474*G0_0_1 - 1.676190476190474*G0_1_0 - 0.101587301587299*G0_1_1;
    A[218] = 1.286772486772486*G0_0_1 + 1.286772486772486*G0_1_0 + 0.135449735449734*G0_1_1;
    A[219] = 0.135449735449735*G0_0_0 + 0.474074074074075*G0_0_1 + 0.474074074074073*G0_1_0 + 0.812698412698413*G0_1_1;
    A[220] = (-0.101587301587302)*G0_0_0 - 0.050793650793652*G0_0_1 - 0.050793650793652*G0_1_0 + 0.406349206349205*G0_1_1;
    A[221] = 0.135449735449736*G0_0_0 - 0.338624338624337*G0_0_1 - 0.338624338624337*G0_1_0;
    A[222] = (-0.812698412698413)*G0_0_0 - 2.031746031746031*G0_0_1 - 2.031746031746031*G0_1_0 - 4.063492063492064*G0_1_1;
    A[223] = (-0.812698412698412)*G0_0_0 + 1.219047619047617*G0_0_1 + 1.219047619047618*G0_1_0 - 0.812698412698411*G0_1_1;
    A[224] = 5.688888888888885*G0_0_0 + 2.844444444444445*G0_0_1 + 2.844444444444445*G0_1_0 + 5.688888888888886*G0_1_1;
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson2d_4_cell_integral_1_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson2d_4_cell_integral_1_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson2d_4_cell_integral_1_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      12
    // Number of operations (multiply-add pairs) for geometry tensor:    15
    // Number of operations (multiply-add pairs) for tensor contraction: 210
    // Total number of operations (multiply-add pairs):                  237
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0 = det*w[0][0]*(1.0);
    const double G0_1 = det*w[0][1]*(1.0);
    const double G0_2 = det*w[0][2]*(1.0);
    const double G0_3 = det*w[0][3]*(1.0);
    const double G0_4 = det*w[0][4]*(1.0);
    const double G0_5 = det*w[0][5]*(1.0);
    const double G0_6 = det*w[0][6]*(1.0);
    const double G0_7 = det*w[0][7]*(1.0);
    const double G0_8 = det*w[0][8]*(1.0);
    const double G0_9 = det*w[0][9]*(1.0);
    const double G0_10 = det*w[0][10]*(1.0);
    const double G0_11 = det*w[0][11]*(1.0);
    const double G0_12 = det*w[0][12]*(1.0);
    const double G0_13 = det*w[0][13]*(1.0);
    const double G0_14 = det*w[0][14]*(1.0);
    
    // Compute element tensor
    A[0] = 0.002557319223986*G0_0 - 0.000238095238095*G0_1 - 0.000238095238095*G0_2 - 0.000987654320988*G0_3 - 0.000105820105820*G0_4 - 0.000987654320988*G0_5 + 0.001410934744268*G0_6 - 0.000705467372134*G0_7 + 0.001410934744268*G0_9 - 0.000705467372134*G0_10 + 0.001410934744268*G0_12 - 0.001410934744268*G0_13 - 0.001410934744268*G0_14;
    A[1] = (-0.000238095238095)*G0_0 + 0.002557319223986*G0_1 - 0.000238095238095*G0_2 + 0.001410934744268*G0_3 - 0.000705467372134*G0_4 - 0.000987654320988*G0_6 - 0.000105820105820*G0_7 - 0.000987654320988*G0_8 - 0.000705467372134*G0_10 + 0.001410934744268*G0_11 - 0.001410934744268*G0_12 + 0.001410934744268*G0_13 - 0.001410934744268*G0_14;
    A[2] = (-0.000238095238095)*G0_0 - 0.000238095238095*G0_1 + 0.002557319223986*G0_2 - 0.000705467372134*G0_4 + 0.001410934744268*G0_5 - 0.000705467372134*G0_7 + 0.001410934744268*G0_8 - 0.000987654320988*G0_9 - 0.000105820105820*G0_10 - 0.000987654320988*G0_11 - 0.001410934744268*G0_12 - 0.001410934744268*G0_13 + 0.001410934744268*G0_14;
    A[3] = (-0.000987654320988)*G0_0 + 0.001410934744268*G0_1 + 0.022574955908289*G0_3 - 0.011287477954145*G0_4 + 0.006772486772487*G0_5 + 0.002257495590829*G0_6 + 0.000564373897707*G0_7 + 0.004514991181658*G0_8 + 0.004514991181658*G0_9 - 0.008465608465608*G0_10 + 0.011287477954145*G0_11 - 0.002257495590829*G0_12 + 0.011287477954145*G0_13 + 0.002257495590829*G0_14;
    A[4] = (-0.000105820105820)*G0_0 - 0.000705467372134*G0_1 - 0.000705467372134*G0_2 - 0.011287477954145*G0_3 + 0.027936507936508*G0_4 - 0.011287477954145*G0_5 + 0.000564373897707*G0_6 + 0.000423280423280*G0_7 - 0.008465608465608*G0_8 + 0.000564373897707*G0_9 + 0.000423280423280*G0_10 - 0.008465608465608*G0_11 - 0.006772486772487*G0_12 + 0.003386243386243*G0_13 + 0.003386243386243*G0_14;
    A[5] = (-0.000987654320988)*G0_0 + 0.001410934744268*G0_2 + 0.006772486772487*G0_3 - 0.011287477954145*G0_4 + 0.022574955908289*G0_5 + 0.004514991181658*G0_6 - 0.008465608465608*G0_7 + 0.011287477954145*G0_8 + 0.002257495590829*G0_9 + 0.000564373897707*G0_10 + 0.004514991181658*G0_11 - 0.002257495590829*G0_12 + 0.002257495590829*G0_13 + 0.011287477954145*G0_14;
    A[6] = 0.001410934744268*G0_0 - 0.000987654320988*G0_1 + 0.002257495590829*G0_3 + 0.000564373897707*G0_4 + 0.004514991181658*G0_5 + 0.022574955908289*G0_6 - 0.011287477954145*G0_7 + 0.006772486772487*G0_8 + 0.011287477954145*G0_9 - 0.008465608465608*G0_10 + 0.004514991181658*G0_11 + 0.011287477954145*G0_12 - 0.002257495590829*G0_13 + 0.002257495590829*G0_14;
    A[7] = (-0.000705467372134)*G0_0 - 0.000105820105820*G0_1 - 0.000705467372134*G0_2 + 0.000564373897707*G0_3 + 0.000423280423280*G0_4 - 0.008465608465608*G0_5 - 0.011287477954145*G0_6 + 0.027936507936508*G0_7 - 0.011287477954145*G0_8 - 0.008465608465608*G0_9 + 0.000423280423280*G0_10 + 0.000564373897707*G0_11 + 0.003386243386243*G0_12 - 0.006772486772487*G0_13 + 0.003386243386243*G0_14;
    A[8] = (-0.000987654320988)*G0_1 + 0.001410934744268*G0_2 + 0.004514991181658*G0_3 - 0.008465608465608*G0_4 + 0.011287477954145*G0_5 + 0.006772486772487*G0_6 - 0.011287477954145*G0_7 + 0.022574955908289*G0_8 + 0.004514991181658*G0_9 + 0.000564373897707*G0_10 + 0.002257495590829*G0_11 + 0.002257495590829*G0_12 - 0.002257495590829*G0_13 + 0.011287477954145*G0_14;
    A[9] = 0.001410934744268*G0_0 - 0.000987654320988*G0_2 + 0.004514991181658*G0_3 + 0.000564373897707*G0_4 + 0.002257495590829*G0_5 + 0.011287477954145*G0_6 - 0.008465608465608*G0_7 + 0.004514991181658*G0_8 + 0.022574955908289*G0_9 - 0.011287477954145*G0_10 + 0.006772486772487*G0_11 + 0.011287477954145*G0_12 + 0.002257495590829*G0_13 - 0.002257495590829*G0_14;
    A[10] = (-0.000705467372134)*G0_0 - 0.000705467372134*G0_1 - 0.000105820105820*G0_2 - 0.008465608465608*G0_3 + 0.000423280423280*G0_4 + 0.000564373897707*G0_5 - 0.008465608465608*G0_6 + 0.000423280423280*G0_7 + 0.000564373897707*G0_8 - 0.011287477954145*G0_9 + 0.027936507936508*G0_10 - 0.011287477954145*G0_11 + 0.003386243386243*G0_12 + 0.003386243386243*G0_13 - 0.006772486772487*G0_14;
    A[11] = 0.001410934744268*G0_1 - 0.000987654320988*G0_2 + 0.011287477954145*G0_3 - 0.008465608465608*G0_4 + 0.004514991181658*G0_5 + 0.004514991181658*G0_6 + 0.000564373897707*G0_7 + 0.002257495590829*G0_8 + 0.006772486772487*G0_9 - 0.011287477954145*G0_10 + 0.022574955908289*G0_11 + 0.002257495590829*G0_12 + 0.011287477954145*G0_13 - 0.002257495590829*G0_14;
    A[12] = 0.001410934744268*G0_0 - 0.001410934744268*G0_1 - 0.001410934744268*G0_2 - 0.002257495590829*G0_3 - 0.006772486772487*G0_4 - 0.002257495590829*G0_5 + 0.011287477954145*G0_6 + 0.003386243386243*G0_7 + 0.002257495590829*G0_8 + 0.011287477954145*G0_9 + 0.003386243386243*G0_10 + 0.002257495590829*G0_11 + 0.094814814814814*G0_12 - 0.013544973544973*G0_13 - 0.013544973544974*G0_14;
    A[13] = (-0.001410934744268)*G0_0 + 0.001410934744268*G0_1 - 0.001410934744268*G0_2 + 0.011287477954145*G0_3 + 0.003386243386243*G0_4 + 0.002257495590829*G0_5 - 0.002257495590829*G0_6 - 0.006772486772487*G0_7 - 0.002257495590829*G0_8 + 0.002257495590829*G0_9 + 0.003386243386243*G0_10 + 0.011287477954145*G0_11 - 0.013544973544973*G0_12 + 0.094814814814815*G0_13 - 0.013544973544973*G0_14;
    A[14] = (-0.001410934744268)*G0_0 - 0.001410934744268*G0_1 + 0.001410934744268*G0_2 + 0.002257495590829*G0_3 + 0.003386243386243*G0_4 + 0.011287477954145*G0_5 + 0.002257495590829*G0_6 + 0.003386243386243*G0_7 + 0.011287477954145*G0_8 - 0.002257495590829*G0_9 - 0.006772486772487*G0_10 - 0.002257495590829*G0_11 - 0.013544973544974*G0_12 - 0.013544973544973*G0_13 + 0.094814814814814*G0_14;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson2d_4_form_0: public ufc::form
{
public:

  /// Constructor
  poisson2d_4_form_0() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson2d_4_form_0()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(IndexSum(Product(SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4), 0), MultiIndex((Index(0),), {Index(0): 2})), SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4), 1), MultiIndex((Index(0),), {Index(0): 2}))), MultiIndex((Index(0),), {Index(0): 2})), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }

  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }

  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson2d_4_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_dof_map_0();
        break;
      }
    case 1:
      {
        return new poisson2d_4_dof_map_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_cell_integral_0_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson2d_4_form_1: public ufc::form
{
public:

  /// Constructor
  poisson2d_4_form_1() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson2d_4_form_1()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(Product(Argument(FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4), 0), Coefficient(FiniteElement('Lagrange', Cell('triangle', 1, Space(2)), 4), 0)), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }

  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }

  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson2d_4_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_dof_map_0();
        break;
      }
    case 1:
      {
        return new poisson2d_4_dof_map_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson2d_4_cell_integral_1_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/GenericFunction.h>
#include <dolfin/function/CoefficientAssigner.h>

namespace Poisson2D_4
{

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:

  CoefficientSpace_f(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  ~CoefficientSpace_f()
  {
  }

};

class Form_0_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_0()
  {
  }

};

class Form_0_FunctionSpace_1: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_1(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_1()
  {
  }

};

class Form_0: public dolfin::Form
{
public:

  // Constructor
  Form_0(const dolfin::FunctionSpace& V0, const dolfin::FunctionSpace& V1):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);
    _function_spaces[1] = reference_to_no_delete_pointer(V1);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_0());
  }

  // Constructor
  Form_0(boost::shared_ptr<const dolfin::FunctionSpace> V0, boost::shared_ptr<const dolfin::FunctionSpace> V1):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_0());
  }

  // Destructor
  ~Form_0()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {

    dolfin::error("No coefficients.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {

    dolfin::error("No coefficients.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_0_FunctionSpace_0 TestSpace;
  typedef Form_0_FunctionSpace_1 TrialSpace;

  // Coefficients
};

class Form_1_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_1_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson2d_4_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson2d_4_dof_map_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_1_FunctionSpace_0()
  {
  }

};

typedef CoefficientSpace_f Form_1_FunctionSpace_1;

class Form_1: public dolfin::Form
{
public:

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson2d_4_form_1());
  }

  // Destructor
  ~Form_1()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::error("Invalid coefficient.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::error("Invalid coefficient.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_1_FunctionSpace_0 TestSpace;
  typedef Form_1_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::CoefficientAssigner f;
};

// Class typedefs
typedef Form_0 BilinearForm;
typedef Form_1 LinearForm;
typedef Form_0::TestSpace FunctionSpace;

}

#endif
